@@ -566,17 +566,21 @@ while language_setup_list:
 if not xmas_flag:
     DIALOGUE_ICON_DICT = {
         'system_icon': 'system_icon_64.png',
+        'ready_icon': 'ready_icon_64.png',
     }
 else:
     DIALOGUE_ICON_DICT = {
         'system_icon': 'system_icon_xmas_64.png',
+        'ready_icon': 'ready_icon_64.png',
     }
 
 if not xmas_flag:
     STATUS_ICON_DICT = {
         'default_icon': 'status_default_icon_64.png',
         'check_icon': 'status_check_icon_64.png',
+        'check_live_icon': 'status_check_live_icon_64.png',
         'download_icon': 'status_download_icon_64.png',
+        'download_live_icon': 'status_download_live_icon_64.png',
         'update_icon': 'status_update_icon_64.png',
         'refresh_icon': 'status_refresh_icon_64.png',
         'info_icon': 'status_info_icon_64.png',
@@ -588,7 +592,9 @@ else:
     STATUS_ICON_DICT = {
         'default_icon': 'status_default_icon_xmas_64.png',
         'check_icon': 'status_check_icon_xmas_64.png',
+        'check_live_icon': 'status_check_live_icon_xmas_64.png',
         'download_icon': 'status_download_icon_xmas_64.png',
+        'download_live_icon': 'status_download_live_icon_xmas_64.png',
         'update_icon': 'status_update_icon_xmas_64.png',
         'refresh_icon': 'status_refresh_icon_xmas_64.png',
         'info_icon': 'status_info_icon_xmas_64.png',
@@ -606,6 +612,8 @@ TOOLBAR_ICON_DICT = {
     'tool_download_small': 'download_small.png',
     'tool_folder_large': 'folder_large.png',
     'tool_folder_small': 'folder_small.png',
+    'tool_hide_large': 'hide_large.png',
+    'tool_hide_small': 'hide_small.png',
     'tool_playlist_large': 'playlist_large.png',
     'tool_playlist_small': 'playlist_small.png',
     'tool_quit_large': 'quit_large.png',
@@ -676,6 +684,10 @@ SMALL_ICON_DICT = {
     'arrow_up_small': 'arrow_up.png',
     'arrow_down_small': 'arrow_down.png',
     'check_small': 'check.png',
+    'debut_now_small': 'debut_now.png',
+    'debut_wait_small': 'debut_wait.png',
+    'delete_small': 'delete.png',
+    'dl_options_small': 'dl_options.png',
     'download_small': 'download.png',
     'error_small': 'error.png',
     'folder_black_small': 'folder_black.png',
@@ -683,14 +695,40 @@ SMALL_ICON_DICT = {
     'folder_green_small': 'folder_green.png',
     'folder_red_small': 'folder_red.png',
     'have_file_small': 'have_file.png',
+    'live_now_small': 'live_now.png',
+    'live_wait_small': 'live_wait.png',
     'no_file_small': 'no_file.png',
-    'stream_live_small': 'stream_live.png',
-    'stream_wait_small': 'stream_wait.png',
     'system_error_small': 'system_error.png',
     'system_warning_small': 'system_warning.png',
     'warning_small': 'warning.png',
 }
 
+THUMB_ICON_DICT = {
+    'thumb_none_tiny': 'thumb_none_tiny.png',
+    'thumb_none_small': 'thumb_none_small.png',
+    'thumb_none_medium': 'thumb_none_medium.png',
+    'thumb_none_large': 'thumb_none_large.png',
+    'thumb_none_enormous': 'thumb_none_enormous.png',
+
+    'thumb_left_tiny': 'thumb_left_tiny.png',
+    'thumb_left_small': 'thumb_left_small.png',
+    'thumb_left_medium': 'thumb_left_medium.png',
+    'thumb_left_large': 'thumb_left_large.png',
+    'thumb_left_enormous': 'thumb_left_enormous.png',
+
+    'thumb_right_tiny': 'thumb_right_tiny.png',
+    'thumb_right_small': 'thumb_right_small.png',
+    'thumb_right_medium': 'thumb_right_medium.png',
+    'thumb_right_large': 'thumb_right_large.png',
+    'thumb_right_enormous': 'thumb_right_enormous.png',
+
+    'thumb_both_tiny': 'thumb_both_tiny.png',
+    'thumb_both_small': 'thumb_both_small.png',
+    'thumb_both_medium': 'thumb_both_medium.png',
+    'thumb_both_large': 'thumb_both_large.png',
+    'thumb_both_enormous': 'thumb_both_enormous.png',
+}
+
 EXTERNAL_ICON_DICT = {
     'ytdl-gui': 'youtube-dl-gui.png',
 }
@@ -700,6 +738,7 @@ STOCK_ICON_DICT = {
     'stock_add': 'add_small.png',
     'stock_cancel': 'cancel_small.png',
     'stock_delete': 'delete_small.png',
+    'stock_execute': 'ffmpeg_small.png',
     'stock_find': 'find_small.png',
     'stock_go_back': 'go_back_small.png',
     'stock_go_down': 'go_down_small.png',
@@ -713,6 +752,7 @@ STOCK_ICON_DICT = {
     'stock_media_stop': 'media_stop_small.png',
     'stock_open': 'open_small.png',
     'stock_properties': 'properties_large.png',
+    'stock_redo': 'resort_small.png',           # Used for a sorting button
     'stock_refresh': 'refresh_small.png',
     'stock_show_filter': 'show_filter_small.png',
     'stock_spell_check': 'spell_check_small.png',
@@ -764,10 +804,10 @@ def do_translate(config_flag=False):
 
     global MAIN_STAGE_QUEUED, MAIN_STAGE_ACTIVE, MAIN_STAGE_PAUSED, \
     MAIN_STAGE_COMPLETED, MAIN_STAGE_ERROR, ACTIVE_STAGE_PRE_PROCESS, \
-    ACTIVE_STAGE_DOWNLOAD, ACTIVE_STAGE_POST_PROCESS, ACTIVE_STAGE_CHECKING, \
-    COMPLETED_STAGE_FINISHED, COMPLETED_STAGE_WARNING, \
-    COMPLETED_STAGE_ALREADY, ERROR_STAGE_ERROR, ERROR_STAGE_STOPPED, \
-    ERROR_STAGE_ABORT
+    ACTIVE_STAGE_DOWNLOAD, ACTIVE_STAGE_POST_PROCESS, ACTIVE_STAGE_CAPTURE, \
+    ACTIVE_STAGE_MERGE, ACTIVE_STAGE_CHECKING, COMPLETED_STAGE_FINISHED, \
+    COMPLETED_STAGE_WARNING, COMPLETED_STAGE_ALREADY, ERROR_STAGE_ERROR, \
+    ERROR_STAGE_STOPPED, ERROR_STAGE_ABORT
 
     global TIME_METRIC_TRANS_DICT
 
@@ -822,6 +862,8 @@ def do_translate(config_flag=False):
     ACTIVE_STAGE_PRE_PROCESS = _('Pre-processing')
     ACTIVE_STAGE_DOWNLOAD = _('Downloading')
     ACTIVE_STAGE_POST_PROCESS = _('Post-processing')
+    ACTIVE_STAGE_CAPTURE = _('Capturing')               # Used by YTSC
+    ACTIVE_STAGE_MERGE = _('Merging')                   # Used by YTSC
     ACTIVE_STAGE_CHECKING = _('Checking')
     # Sub-stages of the 'Completed' stage
     COMPLETED_STAGE_FINISHED = _('Finished')
