@@ -44,16 +44,13 @@ import utils
 from mainapp import _
 
 
-# Debugging flag (calls utils.debug_time at the start of every function)
-DEBUG_FUNC_FLAG = False
-
-
 # Classes
 
 
 class UpdateManager(threading.Thread):
 
-    """Called by mainapp.TartubeApp.update_manager_start().
+    """Called by mainapp.TartubeApp.update_manager_start() or
+    .update_manager_start_from_wizwin().
 
     Python class to create a system child process, to do one of two jobs:
 
@@ -71,16 +68,16 @@ class UpdateManager(threading.Thread):
         update_type (str): 'ffmpeg' to install FFmpeg (on MS Windows only), or
             'ytdl' to install/update youtube-dl
 
+        wiz_win_obj (wizwin.SetupWizWin or None): The calling setup wizard
+            window (if set, the main window doesn't exist yet)
+
     """
 
 
     # Standard class methods
 
 
-    def __init__(self, app_obj, update_type):
-
-        if DEBUG_FUNC_FLAG:
-            utils.debug_time('uop 83 __init__')
+    def __init__(self, app_obj, update_type, wiz_win_obj=None):
 
         super(UpdateManager, self).__init__()
 
@@ -88,6 +85,9 @@ class UpdateManager(threading.Thread):
         # -----------------------
         # The mainapp.TartubeApp object
         self.app_obj = app_obj
+        # The calling setup wizard window (if set, the main window doesn't
+        #   exist yet)
+        self.wiz_win_obj = wiz_win_obj
 
         # This object reads from the child process STDOUT and STDERR in an
         #   asynchronous way
@@ -139,9 +139,6 @@ class UpdateManager(threading.Thread):
         Initiates the download.
         """
 
-        if DEBUG_FUNC_FLAG:
-            utils.debug_time('uop 143 run')
-
         if self.update_type == 'ffmpeg':
             self.install_ffmpeg()
         else:
@@ -163,9 +160,6 @@ class UpdateManager(threading.Thread):
 
         """
 
-        if DEBUG_FUNC_FLAG:
-            utils.debug_time('uop 167 create_child_process')
-
         info = preexec = None
 
         if os.name == 'nt':
@@ -206,12 +200,8 @@ class UpdateManager(threading.Thread):
         application with the result of the update (success or failure).
         """
 
-        if DEBUG_FUNC_FLAG:
-            utils.debug_time('uop 210 install_ffmpeg')
-
         # Show information about the update operation in the Output Tab
-        self.app_obj.main_win_obj.output_tab_write_stdout(
-            1,
+        self.install_ffmpeg_write_output(
             _('Starting update operation, installing FFmpeg'),
         )
 
@@ -227,10 +217,9 @@ class UpdateManager(threading.Thread):
         )
 
         # Show the system command in the Output Tab
-        space = ' '
-        self.app_obj.main_win_obj.output_tab_write_system_cmd(
-            1,
-            space.join( ['pacman', '-S', binary, '--noconfirm'] ),
+        self.install_ffmpeg_write_output(
+            ' '.join( ['pacman', '-S', binary, '--noconfirm'] ),
+            True,                   # A system command, not a message
         )
 
         # So that we can read from the child process STDOUT and STDERR, attach
@@ -256,11 +245,9 @@ class UpdateManager(threading.Thread):
 
                 if stdout:
 
-                    # Show command line output in the Output Tab
-                    self.app_obj.main_win_obj.output_tab_write_stdout(
-                        1,
-                        stdout,
-                    )
+                    # Show command line output in the Output Tab (or wizard
+                    #   window textview)
+                    self.install_ffmpeg_write_output(stdout)
 
         # The child process has finished
         while not self.stderr_queue.empty():
@@ -277,11 +264,9 @@ class UpdateManager(threading.Thread):
 
                 self.stderr_list.append(stderr)
 
-                # Show command line output in the Output Tab
-                self.app_obj.main_win_obj.output_tab_write_stdout(
-                    1,
-                    stderr,
-                )
+                # Show command line output in the Output Tab (or wizard window
+                #   textview)
+                self.install_ffmpeg_write_output(stderr)
 
         # (Generate our own error messages for debugging purposes, in certain
         #   situations)
@@ -300,11 +285,8 @@ class UpdateManager(threading.Thread):
         if not self.stderr_list:
             self.success_flag = True
 
-        # Show a confirmation in the the Output Tab
-        self.app_obj.main_win_obj.output_tab_write_stdout(
-            1,
-            _('Update operation finished'),
-        )
+        # Show a confirmation in the the Output Tab (or wizard window textview)
+        self.install_ffmpeg_write_output(_('Update operation finished'))
 
         # Let the timer run for a few more seconds to prevent Gtk errors (for
         #   systems with Gtk < 3.24)
@@ -314,6 +296,35 @@ class UpdateManager(threading.Thread):
         )
 
 
+    def install_ffmpeg_write_output(self, msg, system_cmd_flag=False):
+
+        """Called by self.install_ffmpeg().
+
+        Writes a message to the Output Tab (or to the setup wizard window, if
+        called from there).
+
+        Args:
+
+            msg (str): The message to display
+
+            system_cmd_flag (bool): If True, display system commands in a
+                different colour in the Output Tab (ignored when writing in
+                the setup wizard window)
+
+        """
+
+        if not self.wiz_win_obj:
+
+            if not system_cmd_flag:
+                self.app_obj.main_win_obj.output_tab_write_stdout(1, msg)
+            else:
+                self.app_obj.main_win_obj.output_tab_write_system_cmd(1, msg)
+
+        else:
+
+            self.wiz_win_obj.ffmpeg_page_write(msg)
+
+
     def install_ytdl(self):
 
         """Called by self.run().
@@ -326,13 +337,11 @@ class UpdateManager(threading.Thread):
         application with the result of the update (success or failure).
         """
 
-        if DEBUG_FUNC_FLAG:
-            utils.debug_time('uop 330 install_ytdl')
-
-        # Show information about the update operation in the Output Tab
-        self.app_obj.main_win_obj.output_tab_write_stdout(
-            1,
-            _('Starting update operation, installing/updating youtube-dl'),
+        # Show information about the update operation in the Output Tab (or in
+        #   the setup wizard window, if called from there)
+        downloader = self.app_obj.get_downloader(self.wiz_win_obj)
+        self.install_ytdl_write_output(
+            _('Starting update operation, installing/updating ' + downloader),
         )
 
         # Prepare the system command
@@ -341,21 +350,32 @@ class UpdateManager(threading.Thread):
         #   depending on how it was installed
         # (For example, if youtube-dl was installed via pip, then it must be
         #   updated via pip)
-        cmd_list \
-        = self.app_obj.ytdl_update_dict[self.app_obj.ytdl_update_current]
+        if self.wiz_win_obj \
+        and self.wiz_win_obj.ytdl_update_current is not None:
+            ytdl_update_current = self.wiz_win_obj.ytdl_update_current
+        else:
+            ytdl_update_current = self.app_obj.ytdl_update_current
 
-        # Convert a path beginning with ~ (not on MS Windows)
-        if os.name != 'nt':
-            cmd_list[0] = re.sub('^\~', os.path.expanduser('~'), cmd_list[0])
+        cmd_list = self.app_obj.ytdl_update_dict[ytdl_update_current]
+
+        mod_list = []
+        for arg in cmd_list:
+
+            # Substitute in the fork, if one is specified
+            arg = self.app_obj.check_downloader(arg, self.wiz_win_obj)
+            # Convert a path beginning with ~ (not on MS Windows)
+            if os.name != 'nt':
+                arg = re.sub('^\~', os.path.expanduser('~'), arg)
+
+            mod_list.append(arg)
 
         # Create a new child process using that command
-        self.create_child_process(cmd_list)
+        self.create_child_process(mod_list)
 
         # Show the system command in the Output Tab
-        space = ' '
-        self.app_obj.main_win_obj.output_tab_write_system_cmd(
-            1,
-            space.join(cmd_list),
+        self.install_ytdl_write_output(
+            ' '.join(mod_list),
+            True,                   # A system command, not a message
         )
 
         # So that we can read from the child process STDOUT and STDERR, attach
@@ -390,19 +410,22 @@ class UpdateManager(threading.Thread):
                     # "The script youtube-dl is installed in '...' which is not
                     #   on PATH. Consider adding this directory to PATH..."
                     if re.search('It looks like you installed', stdout) \
-                    or re.search('The script youtube-dl is installed', stdout):
+                    or re.search(
+                        'The script ' + downloader + ' is installed',
+                        stdout,
+                    ):
                         self.stderr_list.append(stdout)
+
                     else:
+
                         # Try to intercept the new version number for
                         #   youtube-dl
                         self.intercept_version_from_stdout(stdout)
                         self.stdout_list.append(stdout)
 
-                    # Show command line output in the Output Tab
-                    self.app_obj.main_win_obj.output_tab_write_stdout(
-                        1,
-                        stdout,
-                    )
+                    # Show command line output in the Output Tab (or wizard
+                    #   window textview)
+                    self.install_ytdl_write_output(stdout)
 
         # The child process has finished
         while not self.stderr_queue.empty():
@@ -429,43 +452,35 @@ class UpdateManager(threading.Thread):
                 and not re.search('You should consider upgrading', stderr):
                     self.stderr_list.append(stderr)
 
-                # Show command line output in the Output Tab
-                self.app_obj.main_win_obj.output_tab_write_stdout(
-                    1,
-                    stderr,
-                )
+                # Show command line output in the Output Tab (or wizard window
+                #   textview)
+                self.install_ytdl_write_output(stderr)
 
         # (Generate our own error messages for debugging purposes, in certain
         #   situations)
         if self.child_process is None:
 
-            msg = _('youtube-dl update did not start')
+            msg = _('Update did not start')
+
             self.stderr_list.append(msg)
-            self.app_obj.main_win_obj.output_tab_write_stdout(
-                1,
-                msg,
-            )
+            self.install_ytdl_write_output(msg)
 
         elif self.child_process.returncode > 0:
 
             msg = _('Child process exited with non-zero code: {}').format(
                 self.child_process.returncode,
             )
-            self.app_obj.main_win_obj.output_tab_write_stdout(
-                1,
-                msg,
-            )
+
+            self.stderr_list.append(msg)
+            self.install_ytdl_write_output(msg)
 
         # Operation complete. self.success_flag is checked by
         #   mainapp.TartubeApp.update_manager_finished
         if not self.stderr_list:
             self.success_flag = True
 
-        # Show a confirmation in the the Output Tab
-        self.app_obj.main_win_obj.output_tab_write_stdout(
-            1,
-            _('Update operation finished'),
-        )
+        # Show a confirmation in the the Output Tab (or wizard window textview)
+        self.install_ytdl_write_output(_('Update operation finished'))
 
         # Let the timer run for a few more seconds to prevent Gtk errors (for
         #   systems with Gtk < 3.24)
@@ -475,6 +490,35 @@ class UpdateManager(threading.Thread):
         )
 
 
+    def install_ytdl_write_output(self, msg, system_cmd_flag=False):
+
+        """Called by self.install_ytdl().
+
+        Writes a message to the Output Tab (or to the setup wizard window, if
+        called from there).
+
+        Args:
+
+            msg (str): The message to display
+
+            system_cmd_flag (bool): If True, display system commands in a
+                different colour in the Output Tab (ignored when writing in
+                the setup wizard window)
+
+        """
+
+        if not self.wiz_win_obj:
+
+            if not system_cmd_flag:
+                self.app_obj.main_win_obj.output_tab_write_stdout(1, msg)
+            else:
+                self.app_obj.main_win_obj.output_tab_write_system_cmd(1, msg)
+
+        else:
+
+            self.wiz_win_obj.downloader_page_write(msg)
+
+
     def intercept_version_from_stdout(self, stdout):
 
         """Called by self.install_yt_dl() only.
@@ -488,11 +532,8 @@ class UpdateManager(threading.Thread):
 
         """
 
-        if DEBUG_FUNC_FLAG:
-            utils.debug_time('uop 489 intercept_version_from_stdout')
-
         substring = re.search(
-            'Requirement already up\-to\-date.*\(([\d\.]+)\)\s*$',
+            'Requirement already up\-to\-date.*\(([^\(\)]+)\)\s*$',
             stdout,
         )
 
@@ -501,7 +542,7 @@ class UpdateManager(threading.Thread):
 
         else:
             substring = re.search(
-                'Successfully installed youtube\-dl\-([\d\.]+)\s*$',
+                'Successfully installed youtube\-dl\-([^\(\)]+)\s*$',
                 stdout,
             )
 
@@ -525,9 +566,6 @@ class UpdateManager(threading.Thread):
 
         """
 
-        if DEBUG_FUNC_FLAG:
-            utils.debug_time('uop 526 is_child_process_alive')
-
         if self.child_process is None:
             return False
 
@@ -544,9 +582,6 @@ class UpdateManager(threading.Thread):
         Terminates the child process.
         """
 
-        if DEBUG_FUNC_FLAG:
-            utils.debug_time('uop 545 stop_update_operation')
-
         if self.is_child_process_alive():
 
             if os.name == 'nt':
