@@ -36,12 +36,15 @@ import shutil
 import subprocess
 import sys
 import textwrap
+import time
 
 
 # Import our modules
 import formats
 import mainapp
 import media
+# Use same gettext translations
+from mainapp import _
 
 
 # Functions
@@ -383,12 +386,7 @@ def convert_path_to_temp(app_obj, old_path, move_flag=False):
     # Move the file now, if the calling code requires that
     if move_flag:
 
-        # (On MSWin, can't do os.rename if the destination file already exists)
-        if os.path.isfile(new_path):
-            os.remove(new_path)
-
-        # (os.rename sometimes fails on external hard drives; this is safer)
-        shutil.move(old_path, new_path)
+        rename_file(app_obj, old_path, new_path)
 
     # Return the converted file path
     return new_path
@@ -673,6 +671,167 @@ def disk_get_used_space(path, bytes_flag=False):
         return used_bytes
 
 
+def extract_livestream_data(stderr):
+
+    """Called by downloads.JSONFetcher.do_fetch() and
+    MiniJSONFetcher.do_fetch().
+
+    For some reason, YouTube messages giving the (approximate) start time of a
+    livestream are written to STDERR.
+
+    Extracts various data and returns it.
+
+    Args:
+
+        stderr (text): A standard YouTube message
+
+    Return values:
+
+        If extraction is successful, returns a dictionary of three values:
+            live_msg (str): Text that can be displayed in the Video Catalogue
+            live_time (int): Approximate time (matching time.time()) at which
+                the livestream is due to start
+            live_debut_flag (bool): True for a YouTube 'premiere' video, False
+                for an ordinary livestream
+
+        If extraction fails, returns an empty list
+
+    """
+
+    # This live event will begin in a few moments.
+    match_list = re.search(
+        'This live event will begin in a few moments',
+        stderr,
+    )
+
+    if match_list:
+
+        return {
+            'live_msg': _('Live soon'),
+            'live_time': int(time.time()),
+            'live_debut_flag': False,
+        }
+
+    # Premiere will begin shortly
+    match_list = re.search(
+        'Premiere will begin shortly',
+        stderr,
+    )
+
+    if match_list:
+
+        return {
+            'live_msg': _('Debut soon'),
+            'live_time': int(time.time()),
+            'live_debut_flag': True,
+        }
+
+    # This live event will begin in N minutes.
+    match_list = re.search(
+        'This live event will begin in (\d+) minute',
+        stderr,
+    )
+
+    if match_list:
+
+        group_list = match_list.groups()
+        number = int(group_list[0])
+
+        return {
+            'live_msg': _('Live in {0} minutes').format(group_list[0]),
+            'live_time': int(time.time()) + (number * 60),
+            'live_debut_flag': False,
+        }
+
+    # Premieres in N minutes
+    match_list = re.search(
+        'Premieres in (\d+) minute',
+        stderr,
+    )
+
+    if match_list:
+
+        group_list = match_list.groups()
+        number = int(group_list[0])
+
+        return {
+            'live_msg': _('Debut in {0} minutes').format(group_list[0]),
+            'live_time': int(time.time()) + (number * 60),
+            'live_debut_flag': True,
+        }
+
+    # This live event will begin in N hours.
+    match_list = re.search(
+        'This live event will begin in (\d+) hour',
+        stderr,
+    )
+
+    if match_list:
+
+        group_list = match_list.groups()
+        number = int(group_list[0])
+
+        return {
+            'live_msg': _('Live in {0} hours').format(group_list[0]),
+            'live_time': int(time.time()) + (number * 3600),
+            'live_debut_flag': False,
+        }
+
+    # Premieres in N hours
+    match_list = re.search(
+        'Premieres in (\d+) hour',
+        stderr,
+    )
+
+    if match_list:
+
+        group_list = match_list.groups()
+        number = int(group_list[0])
+
+        return {
+            'live_msg': _('Debut in {0} hours').format(group_list[0]),
+            'live_time': int(time.time()) + (number * 3600),
+            'live_debut_flag': True,
+        }
+
+    # This live event will begin in N days.
+    match_list = re.search(
+        'This live event will begin in (\d+) day',
+        stderr,
+    )
+
+    if match_list:
+
+        group_list = match_list.groups()
+        number = int(group_list[0])
+
+        return {
+            'live_msg': _('Live in {0} days').format(group_list[0]),
+            'live_time': int(time.time()) + (number * 86400),
+            'live_debut_flag': False,
+        }
+
+    # Premieres in N days
+    match_list = re.search(
+        'Premieres in (\d+) day',
+        stderr,
+    )
+
+    if match_list:
+
+        group_list = match_list.groups()
+        number = int(group_list[0])
+
+        return {
+            'live_msg': _('Debut in {0} days').format(group_list[0]),
+            'live_time': int(time.time()) + (number * 86400),
+            'live_debut_flag': True,
+        }
+
+    # Not a livestream, return an empty dictionary
+    return {}
+
+
 def find_available_name(app_obj, old_name, min_value=2, max_value=9999):
 
     """Can be called by anything.
@@ -773,48 +932,68 @@ def find_thumbnail(app_obj, video_obj, temp_dir_flag=False):
 
     """
 
-    for ext in formats.IMAGE_FORMAT_LIST:
+    if video_obj.dummy_flag:
 
-        # Look in Tartube's permanent data directory
-        normal_path = video_obj.check_actual_path_by_ext(app_obj, ext)
-        if normal_path is not None:
-            return normal_path
+        # Special case: 'dummy' video objects (those downloaded in the Classic
+        #   Mode tab) use different IVs
+        if video_obj.dummy_path is None:
+            return None
 
-        elif temp_dir_flag:
+        file_name, file_ext = os.path.splitext(video_obj.dummy_path)
+        for this_ext in formats.IMAGE_FORMAT_LIST:
 
-            # Look in temporary data directory
-            data_dir_len = len(app_obj.downloads_dir)
+            thumb_path = file_name + this_ext
+            if os.path.isfile(thumb_path):
+                return thumb_path
 
-            temp_path = video_obj.get_actual_path_by_ext(app_obj, ext)
-            temp_path = app_obj.temp_dl_dir + temp_path[data_dir_len:]
-            if os.path.isfile(temp_path):
-                return temp_path
+        # No matching thumbnail found
+        return None
 
-    # Catch YouTube .jpg thumbnails, in the form .jpg?...
-    # v2.2.005 The glob.glob() call crashes on certain videos. I'm not sure
-    #   why, but we can circumvent the crash with try...except
-    normal_path = video_obj.get_actual_path_by_ext(app_obj, '.jpg*')
-    try:
-        for glob_path in glob.glob(normal_path):
-            if os.path.isfile(glob_path):
-                return glob_path
-    except:
-        pass
+    else:
 
-    if temp_dir_flag:
+        # All other media.Video objects
+        for ext in formats.IMAGE_FORMAT_LIST:
 
-        temp_path = video_obj.get_actual_path_by_ext(app_obj, '.jpg*')
-        temp_path = app_obj.temp_dl_dir + temp_path[data_dir_len:]
+            # Look in Tartube's permanent data directory
+            normal_path = video_obj.check_actual_path_by_ext(app_obj, ext)
+            if normal_path is not None:
+                return normal_path
 
+            elif temp_dir_flag:
+
+                # Look in temporary data directory
+                data_dir_len = len(app_obj.downloads_dir)
+
+                temp_path = video_obj.get_actual_path_by_ext(app_obj, ext)
+                temp_path = app_obj.temp_dl_dir + temp_path[data_dir_len:]
+                if os.path.isfile(temp_path):
+                    return temp_path
+
+        # Catch YouTube .jpg thumbnails, in the form .jpg?...
+        # v2.2.005 The glob.glob() call crashes on certain videos. I'm not sure
+        #   why, but we can circumvent the crash with try...except
+        normal_path = video_obj.get_actual_path_by_ext(app_obj, '.jpg*')
         try:
-            for glob_path in glob.glob(temp_path):
+            for glob_path in glob.glob(normal_path):
                 if os.path.isfile(glob_path):
                     return glob_path
         except:
             pass
 
-    # No matching thumbnail found
-    return None
+        if temp_dir_flag:
+
+            temp_path = video_obj.get_actual_path_by_ext(app_obj, '.jpg*')
+            temp_path = app_obj.temp_dl_dir + temp_path[data_dir_len:]
+
+            try:
+                for glob_path in glob.glob(temp_path):
+                    if os.path.isfile(glob_path):
+                        return glob_path
+            except:
+                pass
+
+        # No matching thumbnail found
+        return None
 
 
 def find_thumbnail_restricted(app_obj, video_obj):
@@ -1045,6 +1224,13 @@ divert_mode=None):
     if app_obj.ytdl_write_verbose_flag:
         options_list.append('--verbose')
 
+    # Specify a standard location for the cookie jar (otherwise youtube-dl
+    #   will write it to ../tartube/tartube)
+    options_list.append('--cookies')
+    options_list.append(
+        os.path.abspath(os.path.join(app_obj.data_dir, 'cookies.txt')),
+    )
+
     # Supply youtube-dl with the path to the ffmpeg/avconv binary, if the
     #   user has provided one
     # If both paths have been set, prefer ffmpeg, unless the 'prefer_avconv'
@@ -1137,6 +1323,40 @@ def get_options_manager(app_obj, media_data_obj):
         return app_obj.general_options_obj
 
 
+def get_ytsc_scripts(app_obj):
+
+    """Called by downloads.DownloadWorker.check_ytsc().
+
+    Returns a list of paths to the scripts comprising Youtube Stream Capture.
+
+    Args:
+
+        app_obj (mainapp.TartubeApp): The main application
+
+    Return values:
+
+        A list of three items, giving paths to the scripts
+            [ youtube_stream_capture.py, merge_v1.py, merge_v2.py ]
+
+        If the path to YTSC's directory is unknown, returns three None values
+            instead
+
+    """
+
+    if app_obj.ytsc_path is None:
+        return []
+
+    else:
+
+        script_dir, filename = os.path.split(app_obj.ytsc_path)
+
+        return [
+            app_obj.ytsc_path,
+            os.path.abspath(os.path.join(script_dir, 'merge_v1.py')),
+            os.path.abspath(os.path.join(script_dir, 'merge_v2.py')),
+        ]
+
+
 def is_youtube(url):
 
     """Can be called by anything.
@@ -1219,7 +1439,7 @@ def move_thumbnail_to_subdir(app_obj, video_obj):
 
     """
 
-    path_list = utils.find_thumbnail_restricted(app_obj, video_obj)
+    path_list = find_thumbnail_restricted(app_obj, video_obj)
     if path_list:
 
         main_path = os.path.abspath(
@@ -1253,7 +1473,7 @@ def move_thumbnail_to_subdir(app_obj, video_obj):
                 pass
 
 
-def open_file(uri):
+def open_file(app_obj, uri):
 
     """Can be called by anything.
 
@@ -1262,23 +1482,39 @@ def open_file(uri):
 
     Args:
 
+        app_obj (mainapp.TartubeApp): The main application
+
         uri (str): The URI to open
 
     """
 
     if sys.platform == "win32":
-        os.startfile(uri)
+
+        try:
+            os.startfile(uri)
+        except:
+            app_obj.system_error(
+                501,
+                'Could not open \'' + str(uri) + '\'',
+            )
+
     else:
-        opener ="open" if sys.platform == "darwin" else "xdg-open"
-        subprocess.call([opener, uri])
+
+        opener = "open" if sys.platform == "darwin" else "xdg-open"
+        # (Assume a return code of 0 on success)
+        if subprocess.call([opener, uri]):
+            app_obj.system_error(
+                502,
+                'Could not open \'' + str(uri) + '\'',
+            )
 
 
-def parse_ytdl_options(options_string):
+def parse_options(options_string):
 
     """Called by options.OptionsParser.parse() or info.InfoManager.run().
 
-    Also called by process.ProcessManager.__init__, to parse FFmpeg command-
-    line options on the same basis.
+    Also called by ffmpeg_tartube.FFmpegOptionsManager.get_system_cmd() to
+    parse FFmpeg command-line options on the same basis.
 
     Parses the 'extra_cmd_string' option, which can contain arguments inside
     double quotes "..." (arguments that can therefore contain whitespace)
@@ -1328,6 +1564,38 @@ def parse_ytdl_options(options_string):
     return return_list
 
 
+def rename_file(app_obj, old_path, new_path):
+
+    """Can be called by anything.
+
+    Renames (mmoves) a file. (Is not usually called for directories, but could
+    be.)
+
+    Args:
+
+        old_path (str): Full path to the file to be renamed
+
+        new_path (str): Full path to the renamed file
+
+    """
+
+    try:
+
+        # (On MSWin, can't do os.rename if the destination file already exists)
+        if os.path.isfile(new_path):
+            os.remove(new_path)
+
+        # (os.rename sometimes fails on external hard drives; this is safer)
+        shutil.move(old_path, new_path)
+
+    except:
+
+        app_obj.system_error(
+            503,
+            'Could not rename \'' + str(old_path) + '\'',
+        )
+
+
 def shorten_string(string, num_chars):
 
     """Can be called by anything.
@@ -1353,6 +1621,85 @@ def shorten_string(string, num_chars):
     return string
 
 
+def shorten_string_two_lines(string, num_chars):
+
+    """Can be called by anything.
+
+    Modified version of shorten_string(). Reduces the string to two lines
+    (separated by a newline character). Each line has the specified maximum
+    length. If there is too much text to fit on the second line, truncates it
+    and adds an ellipsis.
+
+    Args:
+
+        string (string): The string to convert
+
+        num_chars (int): The maximum length of the desired string
+
+    Returns:
+
+        The converted string
+
+    """
+
+    if string and len(string) > num_chars:
+
+        line_list = []
+        current_line = ''
+
+        for word in string.split():
+
+            if len(word) > num_chars:
+
+                # To keep the code simple, the algorithm ends here, with this
+                #   word on a separate line. This may produce a return string
+                #   containing only line
+                if current_line != '':
+                    line_list.append(current_line)
+
+                line_list.append(word[0:num_chars] + '...')
+                break
+
+            else:
+
+                if current_line != '':
+                    mod_line = current_line + ' ' + word
+                else:
+                    mod_line = word
+
+                if len(mod_line) > num_chars:
+
+                    if current_line != '':
+                        line_list.append(current_line)
+
+                    current_line = word
+
+                else:
+
+                    current_line = mod_line
+
+        line_list.append(current_line)
+
+        # Dispense with everything but the first two lines
+        line_count = len(line_list)
+        if line_count > 2:
+
+            return line_list[0] + '\n' + line_list[1] + '...'
+
+        elif line_count == 2:
+
+            return line_list[0] + '\n' + line_list[1]
+
+        else:
+
+            return line_list[0]
+
+    else:
+
+        # 'string' is empty or short, so there's no need to split it up at all
+        return string
+
+
 def strip_whitespace(string):
 
     """Can be called by anything.
@@ -1490,14 +1837,16 @@ def tidy_up_long_descrip(string, max_length=80):
 
             else:
 
-                new_list = textwrap.wrap(
-                    line,
+                w = ModTextWrapper(
                     width=max_length,
-                    # Don't split up URLs
                     break_long_words=False,
-                    break_on_hyphens=False,
+                    # Split up URLs on the forward slash character, as well as
+                    #   on hyphen(s)
+                    break_on_hyphens=True,
                 )
 
+                new_list = w.wrap(line)
+
                 for mini_line in new_list:
                     line_list.append(mini_line)
 
@@ -1564,14 +1913,17 @@ split_words_flag=False):
                 line_list.append('')
 
             else:
-                new_list = textwrap.wrap(
-                    line,
+
+                w = ModTextWrapper(
                     width=max_length,
-                    # Don't split up URLs by default
                     break_long_words=split_words_flag,
+                    # Split up URLs on the forward slash character, as well as
+                    #   on hyphen(s)
                     break_on_hyphens=split_words_flag,
                 )
 
+                new_list = w.wrap(line)
+
                 for mini_line in new_list:
                     line_list.append(mini_line)
 
@@ -1617,3 +1969,52 @@ def upper_case_first(string):
     """
 
     return string[0].upper() + string[1:]
+
+
+# Classes
+
+
+class ModTextWrapper(textwrap.TextWrapper):
+
+    """Python class to modify the behaviour of textwrap by Gregory P. Ward.
+
+    If 'break_on_hyphens' is specified, wrap the text on both hyphens and
+    forward slashes. In that way, we can break up long URLs in calls to
+    utils.tidy_up_long_descrip() and utils.tidy_up_long_string().
+
+    """
+
+    def _split(self, text):
+
+        _whitespace = '\t\n\x0b\x0c\r '
+
+        word_punct = r'[\w!"\'&.,?]'
+        letter = r'[^\d\W]'
+        whitespace = r'[%s]' % re.escape(_whitespace)
+        nowhitespace = '[^' + whitespace[1:]
+        mod_wordsep_re = re.compile(r'''
+            ( # any whitespace
+              %(ws)s+
+            | # em-dash between words
+              (?<=%(wp)s) -{2,} (?=\w)
+            | # word, possibly hyphenated
+              %(nws)s+? (?:
+                # hyphenated word, or word with forward slash
+                  [-\/](?: (?<=%(lt)s{2}[-\/]) | (?<=%(lt)s[-\/]%(lt)s[-\/]))
+                  (?= %(lt)s [-\/]? %(lt)s)
+                | # end of word
+                  (?=%(ws)s|\Z)
+                | # em-dash
+                  (?<=%(wp)s) (?=-{2,}\w)
+                )
+            )''' % {'wp': word_punct, 'lt': letter,
+                    'ws': whitespace, 'nws': nowhitespace},
+            re.VERBOSE)
+
+        if self.break_on_hyphens is True:
+            chunks = mod_wordsep_re.split(text)
+        else:
+            chunks = self.wordsep_simple_re.split(text)
+        chunks = [c for c in chunks if c]
+        return chunks
+
